{% load static %}

<script src="{% static 'datatables/js/moment.min.js' %}" type="text/javascript"></script>
<script src="{% static 'datatables/js/jquery-3.3.1.min.js' %}" type="text/javascript"></script>
<script src="{% static 'datatables/js/popper.min.js' %}" type="text/javascript"></script>
<script src="{% static 'datatables/js/bootstrap.min.js' %}" type="text/javascript"></script>
<script src="{% static 'datatables/js/datatables.min.js' %}" type="text/javascript"></script>
<script src="{% static 'datatables/js/datatables.select.min.js' %}" type="text/javascript"></script>
<script src="{% static 'datatables/js/dataTables.dateTime.min.js' %}" type="text/javascript"></script>
<script src="{% static 'datatables/js/django-datatables.js' %}" type="text/javascript"></script>
