{% load static %}

<link rel="stylesheet" href="{% static 'datatables/css/bootstrap.min.css' %}">
<link rel="stylesheet" href="{% static 'datatables/css/datatables.min.css' %}">
<link rel="stylesheet" href="{% static 'datatables/css/select.datatables.min.css' %}">
<link rel="stylesheet" href="{% static 'datatables/css/dataTables.dateTime.min.css' %}">
